#include "jc.h"
#include "jpegreader.h"
#include "jpegwriter.h"

void usage(char *progname)
{
  printf("jc. Verion 0.91.\n");
  printf("Supports additional comments in the common JPEG file.\n");
  printf("Usage: \n\t%s -view (-v) inputfile (outputfile)\n", progname);
  printf("\tor %s -add (-a) inputfile datafile outputfile\n", progname);
  printf("\tor %s -crc (-c) inputfile (outputfile)\n", progname);
  printf("\tor %s -crc_ex (-e) inputfile (outputfile)\n", progname);
  printf("\tor %s -del (-d) inputfile outputfile\n", progname);
}

long getFileSize(char * filename)
{
    int handle = open(filename, O_RDONLY | O_BINARY);
	long fileSize = lseek(handle,0,SEEK_END);
	close(handle);
	return fileSize;	
}

void readData(char *filename, bool printCRC, bool printCRCEx, char *outname)
{
	if((outname != NULL) && (strcasecmp(filename, outname) == 0))	
	{
		printf("Invalid arguments: Source and Out files are the same: %s.\n", outname);
		return;
	}

    JPEGReader rd;
    rd.scanJPEGHeader(filename);    

    if(rd.getStatus())
    {
        if(printCRC)
        {
			unsigned long sz = getFileSize(filename) / 1000L;

            char buff[33];
            rd.printCRC(buff);
            if(outname == NULL) 
			{
				if(printCRCEx)	
					printf("%ldk / %s\n",sz, buff);
				else	
					printf("CRC: %s\n",buff);
			}
            else
            {
                FILE *out = fopen(outname, "w+");
				if(printCRCEx)	
					fprintf(out,"%ldk / %s\n",sz, buff);
				else
					fprintf(out,"CRC: %s\n",buff);
                fclose(out);
            }
        }
        else
        {
            unsigned int len = rd.getDataLength();
            unsigned char *data = rd.getData();

            if(outname == NULL)
            {       
                for(unsigned int i = 0; i < len; i++)
                    putchar(data[i]);
                printf("\n");
            }
            else
            {
                int handle = creat(outname, S_IREAD | S_IWRITE);
                close(handle);
                handle = open(outname, O_WRONLY | O_BINARY);
                if(handle > 0)
                    write(handle, data, len);
                close(handle);      
            }
        }
    }
    else 
        printf("No additional JPEG APP7 data found in file %s.\n", filename);
}

void writeData(char *src_filename, char *data_filename, char *tgt_filename)
{
    JPEGWriter wr;
	if(strcasecmp(src_filename, tgt_filename) == 0)	
	{
		printf("Invalid arguments: Source and Target files are the same: %s.\n", tgt_filename);
		return;
	}

	if((data_filename != NULL) && (strcasecmp(data_filename, tgt_filename) == 0))	
	{
		printf("Invalid arguments: Data and Target files are the same: %s.\n", tgt_filename);
		return;
	}

	if((data_filename != NULL) && (strcasecmp(data_filename, src_filename) == 0))	
	{
		printf("Invalid arguments: Source and Data files are the same: %s.\n", src_filename);
		return;
	}

    wr.writeJPEGMarkerContent(src_filename, data_filename, tgt_filename);
    if(wr.getStatus())
        printf("Operation successfully complete.\n");
}

void readData(char *filename, bool printCRC, bool printCRCEx)
{
    readData(filename, printCRC, printCRCEx, NULL); 
}

int main(int argc, char* argv[])
{
    if(argc < 2)
    {
        usage(argv[0]);
        return 1;   
    }

    if(!strcmp(argv[1],"-view")||!strcmp(argv[1],"-v"))
    {
        if(argc == 3)
            readData(argv[2], false, false);
        else
            readData(argv[2], false, false, argv[3]);
    }
    else
    if(!strcmp(argv[1],"-add")||!strcmp(argv[1],"-a"))
    {
        if(argc >= 5)
            writeData(argv[2], argv[3], argv[4]);
    }
    else
    if(!strcmp(argv[1],"-del")||!strcmp(argv[1],"-d"))
    {
        if(argc >= 4)
            writeData(argv[2], NULL, argv[3]);
    }
    else
    if(!strcmp(argv[1],"-crc")||!strcmp(argv[1],"-c"))
    {
        if(argc == 3)
            readData(argv[2], true, false);
        else
            readData(argv[2], true, false, argv[3]);
    }
	else
    if(!strcmp(argv[1],"-crc_ex")||!strcmp(argv[1],"-e"))
    {
        if(argc == 3)
            readData(argv[2], true, true);
        else
            readData(argv[2], true, true, argv[3]);
    }
    else
    {
        if(argc == 2)
            readData(argv[1], false, false);
        else
            readData(argv[1], false, false, argv[2]);
    }
    return 0;
}
