#ifndef JPEGREADER_H
#define JPEGREADER_H

#include "jpeglog.h"
#include "filereader.h"

#include <time.h>

/*
* JPEG markers consist of one or more 0xFF bytes, followed by a marker
* code byte (which is not an FF).  Here are the marker codes of interest
* in this program.  (See jdmarker.c for a more complete list.)
*/

#define M_SOF0  0xC0        /* Start Of Frame N */
#define M_SOF1  0xC1        /* N indicates which compression process */
#define M_SOF2  0xC2        /* Only SOF0-SOF2 are now in common use */
#define M_SOF3  0xC3
#define M_SOF5  0xC5        /* NB: codes C4 and CC are NOT SOF markers */
#define M_SOF6  0xC6
#define M_SOF7  0xC7
#define M_SOF9  0xC9
#define M_SOF10 0xCA
#define M_SOF11 0xCB
#define M_SOF13 0xCD
#define M_SOF14 0xCE
#define M_SOF15 0xCF
#define M_SOI   0xD8        /* Start Of Image (beginning of datastream) */
#define M_EOI   0xD9        /* End Of Image (end of datastream) */
#define M_SOS   0xDA        /* Start Of Scan (begins compressed data) */
#define M_COM   0xFE        /* COMment */

#define M_APP0  0xE0        /* Application specific marker #0 */
#define M_APP7  0xE7        /* Application specific marker #7 */




#define MAGIC 0x0707

#define VERSION_1 1
#define VERSION_1_HEADER_SIZE 23

#define CURRENT_VERSION VERSION_1


class JPEGReader : public JPEGLog
{
public: 

    JPEGReader();
    ~JPEGReader();

    void scanJPEGHeader(const char * filename);

    bool    getStatus()             const { return m_status; }
    int     getVersion()            const { return m_version; }
    time_t  getDate()               const { return m_date; }

    unsigned int getDataLength()    const { return m_length;}
    unsigned char * getData()       const { return m_data; }    
    
    FileReader * getFileReader(){ return m_reader; }

    void         printCRC(char * buff);

protected:

    unsigned char   read1Byte();
    unsigned int    read2Bytes();
    unsigned long   read4Bytes();

    int     getFirstMarker();
    int     getNextMarker();
    void    skipData();
    void    processMarker();

    void setStatus(bool status) { m_status = status; }

    void setFileReader(FileReader *reader);

    static void printByte(unsigned char byte, char * buff);
    static void printSymbol(unsigned char n, char * buff);

private:

    FileReader      *m_reader;

    unsigned long   m_length;
    unsigned char * m_data;

    int             m_version;  
    bool            m_status;
    time_t          m_date;
    unsigned char   m_crc[16];
    long            m_crc_pos;
};

#endif

